from django.conf import settings
from django.db import models
from django.utils.timezone import now


class RequestLog(models.Model):
    created_at= models.DateTimeField(auto_now_add=True)
    method = models.CharField(max_length=10)
    url = models.TextField()
    request_body = models.TextField(blank=True, default="")
    response_status = models.IntegerField()
    response_body = models.TextField(blank=True, default="")
    start = models.DateTimeField()
    end = models.DateTimeField()
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE, null=True, default=None
    )

    def __str__(self):
        return f"{self.method} {self.url} by {self.user} at {self.start}"

    @property
    def duration(self):
        return self.end - self.start

    @classmethod
    def store(cls, request, response, start):
        end = now()

        try:
            user = None
            if request.user.is_authenticated:
                user = request.user

            try:
                request_body = request.body.decode()
            except UnicodeDecodeError:
                request_body = request.body

            try:
                response_body = (response.content or b"").decode()
            except UnicodeDecodeError:
                response_body = response.content or b""

            return cls.objects.create(
                method=request.method,
                url=request.path,
                request_body=request_body,
                response_status=response.status_code,
                response_body=response_body,
                start=start,
                end=end,
                user=user,
            )
        except Exception:
            logging.exception("an error occured while storing data about api call")
        return None



import logging
import time
from django.utils.timezone import now
from .models import RequestLog
from django.conf import settings

class DbreqrespMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        self.paths = getattr(settings, "DBREQRESP_MIDDLEWARE_PATHS", [])

    def __call__(self, request):
        start = now()
        response = self.get_response(request)
        if any([request.path.startswith(path) for path in self.paths]):
            RequestLog.store(request, response, start)
        return response



from django.contrib import admin

from .models import RequestLog


@admin.register(RequestLog)
class RequestLogAdmin(admin.ModelAdmin):
    list_display = ["start", "user", "duration", "method", "url", "response_status"]
    search_fields = ("url", "response_status")
    list_filter = ["response_status", "start", "user", "method"]



from django.utils.timezone import now

from .models import RequestLog


def dbreqresp(function):
    def wrapper(request, *args, **kwargs):
        start = now()
        response = function(request, *args, **kwargs)
        RequestLog.store(request, response, start)
        return response
    return wrapper



# Generated by Django 6.0.1 on 2026-01-29 10:19

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="RequestLog",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("method", models.CharField(max_length=10)),
                ("url", models.TextField()),
                ("request_body", models.TextField(blank=True, default="")),
                ("response_status", models.IntegerField()),
                ("response_body", models.TextField(blank=True, default="")),
                ("start", models.DateTimeField()),
                ("end", models.DateTimeField()),
                (
                    "user",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]
